﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;


namespace CellStyleSelector
{
    public class StocksViewModel : INotifyPropertyChanged, IDisposable
    {
        #region Members

        private ObservableCollection<StockData> data;
        private Random r = new Random(123345345);
        internal DispatcherTimer timer;
        private bool enableTimer = false;
        private double timerValue;
        private string _ButtonContnt;
        private int noOfUpdates = 500;
        private List<string> StockSymbols = new List<string>();

        private string[] accounts = new string[]
        {
            "AmericanFunds",
            "ChildrenCollegeSavings",
            "DayTrading",
            "RetirementSavings",
            "MountainRanges",
            "FidelityFunds",
            "Mortages",
            "HousingLoans"
        };

        #endregion

        public ObservableCollection<StockData> Stocks
        {
            get { return this.data; }
        }

        #region Constructor

        public StocksViewModel()
        {
            this.data = new ObservableCollection<StockData>();
            this.AddRows(2000);
            this.timer = new DispatcherTimer();
            this.ResetRefreshFrequency(2500);
            timer.Interval = TimeSpan.FromMilliseconds(100);
            timer.Tick += timer_Tick;
        }


        #endregion

        #region Timer and updating code

        public void SetInterval(TimeSpan time)
        {
            this.timer.Interval = time;
        }

        public void StartTimer()
        {
            if (!this.timer.IsEnabled)
            {
                this.timer.Start();
                this.ButtonContnt = "Stop Timer";
            }
        }

        public double TimeSpanValue
        {
            get { return timerValue; }
            set
            {
                timerValue = value;
                this.timer.Interval = TimeSpan.FromMilliseconds(timerValue);
                OnPropertyChanged("TimeSpanValue");
            }
        }

        public string ButtonContnt
        {
            get { return _ButtonContnt; }
            set
            {
                _ButtonContnt = value;
                OnPropertyChanged("ButtonContnt");
            }
        }

        private bool CanButtonClick(object param)
        {
            return true;
        }

        private void ButtonClicked(object param)
        {
            if (ButtonContnt.Equals("Start Timer"))
            {
                this.EnableTimer = true;

                this.StartTimer();
                ButtonContnt = "Stop Timer";
            }
            else
            {
                this.EnableTimer = false;

                this.StopTimer();
                ButtonContnt = "Start Timer";
            }
        }

        public void StopTimer()
        {
            this.timer.Stop();
        }

        private void timer_Tick(object sender, object e)
        {
            int startTime = DateTime.Now.Millisecond;
            ChangeRows(noOfUpdates);
        }

        public bool EnableTimer
        {
            get { return this.enableTimer; }
            set { this.enableTimer = value; }
        }

        private void AddRows(int count)
        {
            for (int i = 0; i < count; ++i)
            {
                var newRec = new StockData();
                newRec.Symbol = ChangeSymbol();
                newRec.Account = ChangeAccount(i);
                newRec.Open = Math.Round(r.NextDouble()*30, 2);
                newRec.LastTrade = Math.Round((1 + r.NextDouble()*50));
                double d = r.NextDouble();
                if (d < .5)
                    newRec.Change = Math.Round(d, 2);
                else
                    newRec.Change = Math.Round(d, 2)*-1;

                newRec.PreviousClose = Math.Round(r.NextDouble()*30, 2);
                newRec.Volume = r.Next();
                data.Add(newRec);
            }
        }

        private String ChangeSymbol()
        {
            StringBuilder builder = new StringBuilder();
            Random random = new Random();
            char ch;

            do
            {
                builder = new StringBuilder();
                for (int i = 0; i < 4; i++)
                {
                    ch = Convert.ToChar(Convert.ToInt32(Math.Floor(26*random.NextDouble() + 65)));
                    builder.Append(ch);
                }

            } while (StockSymbols.Contains(builder.ToString()));

            StockSymbols.Add(builder.ToString());
            return builder.ToString();
        }

        private String ChangeAccount(int index)
        {
            return accounts[index%accounts.Length];
        }

        public void ResetRefreshFrequency(int changesPerTick)
        {
            if (this.timer == null)
            {
                return;
            }

            if (!this.noOfUpdates.Equals(changesPerTick))
            {
                this.StopTimer();
                this.noOfUpdates = changesPerTick;
                if (enableTimer)
                    this.StartTimer();
            }
        }

        public object SelectedItem
        {
            get { return noOfUpdates; }
            set
            {
                noOfUpdates = int.Parse(((ComboBoxItem) value).Content.ToString());
                OnPropertyChanged("SelectedItem");
            }
        }

        public List<int> ComboCollection
        {
            get { return new List<int> {500, 5000, 50000, 500000}; }
        }

        private void ChangeRows(int count)
        {
            if (data.Count < count)
                count = data.Count;
            for (int i = 0; i < count; ++i)
            {
                int recNo = r.Next(data.Count);
                StockData recRow = data[recNo];

                data[recNo].LastTrade = Math.Round((1 + r.NextDouble()*50));

                double d = r.NextDouble();
                if (d < .5)
                    data[recNo].Change = Math.Round(d, 2);
                else
                    data[recNo].Change = Math.Round(d, 2)*-1;
                data[recNo].Open = Math.Round(r.NextDouble()*50, 2);
                data[recNo].PreviousClose = Math.Round(r.NextDouble()*30, 2);
                data[recNo].Volume = r.Next();
            }
        }

        #endregion

        #region IDisposable Members

        public void Dispose()
        {
            if (this.timer != null)
            {
                this.timer.Tick -= timer_Tick;
                this.StopTimer();
            }
        }

        #endregion

        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged(String prop)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(prop));
            }
        }
    }
}
